﻿
using System.IO;
using System.Reflection;
using System.Text;
using System.Xml;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Xml.Extensions;
using OpcLabs.UnitTestLib;

namespace OpcLabs.BaseLibTest.Xml
{
    [TestClass]
    public class XmlWriterExtensionTest
    {
        #region Static Private Methods

        static private void DynamicSerializeTest([CanBeNull] object value, [NotNull] string expectedOutput)
        {
            var stringBuilder = new StringBuilder();
            using (TextWriter textWriter = new StringWriter(stringBuilder))
            {
                var xmlTextWriter = new XmlTextWriter(textWriter);
                xmlTextWriter.DynamicSerialize("TestElementName", "testDefaultNamespace", value);
            }
            string serializedText = stringBuilder.ToString();
            AssertUtilities.WriteLine(serializedText);

            StringAssertUtilities.AreEqual(expectedOutput, serializedText);
        }

        #endregion



        #region Test Methods

        public class TestClass
        {
            // ReSharper disable once InconsistentNaming
            public string field1;

            // ReSharper disable once InconsistentNaming
            public string field2;
        }

        [TestMethod]
        public void DynamicSerializeTest_Int32()
        {
            DynamicSerializeTest(
                1111111111,
#if (NETFRAMEWORK)
                "<TestElementName xmlns=\"testDefaultNamespace\"><TypeFullName xmlns=\"http://schemas.datacontract.org/2004/07/OpcLabs.BaseLib.Xml\">System.Int32</TypeFullName><Instance xmlns=\"http://schemas.datacontract.org/2004/07/OpcLabs.BaseLib.Xml\">1111111111</Instance></TestElementName>");
#else
                "<TestElementName xmlns=\"testDefaultNamespace\"><TypeFullName xmlns=\"http://schemas.datacontract.org/2004/07/OpcLabs.BaseLib.Xml\">System.Int32</TypeFullName><AssemblyFullName xmlns=\"http://schemas.datacontract.org/2004/07/OpcLabs.BaseLib.Xml\">System.Private.CoreLib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=7cec85d7bea7798e</AssemblyFullName><Instance xmlns=\"http://schemas.datacontract.org/2004/07/OpcLabs.BaseLib.Xml\">1111111111</Instance></TestElementName>");
#endif
        }

        [TestMethod]
        public void DynamicSerializeTest_Null()
        {
            DynamicSerializeTest(
                null, 
                "<TestElementName xmlns=\"testDefaultNamespace\" />");
        }


        [TestMethod]
        public void DynamicSerializeTest_TestClass()
        {
            string thisAssemblyName = Assembly.GetExecutingAssembly().FullName;
            DynamicSerializeTest(
                new TestClass { field1 = "testField1", field2 = "testField2" },
                "<TestElementName xmlns=\"testDefaultNamespace\"><TypeFullName xmlns=\"http://schemas.datacontract.org/2004/07/OpcLabs.BaseLib.Xml\">OpcLabs.BaseLibTest.Xml.XmlWriterExtensionTest+TestClass</TypeFullName><AssemblyFullName xmlns=\"http://schemas.datacontract.org/2004/07/OpcLabs.BaseLib.Xml\">" + thisAssemblyName + "</AssemblyFullName><Instance xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns=\"http://schemas.datacontract.org/2004/07/OpcLabs.BaseLib.Xml\"><field1>testField1</field1><field2>testField2</field2></Instance></TestElementName>");
        }

#endregion
    }
}
