﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.OperationModel;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.DataAccess
{
    // ReSharper disable once InconsistentNaming
    public partial class IEasyDAClientTest_BrowseServers
    {
        #region Static Private Methods

        private void BackgroundWorkers_InstantiateAndBrowseServers_WhileSubscribed(int threadCount, int loopCount = 1)
        {
            var client = CreateClient();
            client.SubscribeMultipleItems(new[]
            {
                new EasyDAItemSubscriptionArguments("", LegacyTestDefs.ProgIds.KitServer, "Simulation.Random", 100, ItemChangedCallback, null),
                new EasyDAItemSubscriptionArguments("", LegacyTestDefs.ProgIds.KitServer, "Simulation.Ramp (10 s)", 100, ItemChangedCallback, null)
            });
            for (int i = 0; i < loopCount; i++)
            {
                AssertUtilities.WriteLine(i);
                IEasyDAClientTest_Com.BackgroundWorkersTest(InstantiateAndBrowseServers, threadCount);
            }
            client.UnsubscribeAllItems();
        }

        private void InstantiateAndBrowseServers()
        {
            var client = CreateClient();
            client.BrowseServers("");
        }

        private void Threads_InstantiateAndBrowseServers_WhileSubscribed(int threadCount, int loopCount = 1)
        {
            var client = CreateClient();
            client.SubscribeMultipleItems(new[]
            {
                new EasyDAItemSubscriptionArguments("", LegacyTestDefs.ProgIds.KitServer, "Simulation.Random", 100, ItemChangedCallback, null),
                new EasyDAItemSubscriptionArguments("", LegacyTestDefs.ProgIds.KitServer, "Simulation.Ramp (10 s)", 100, ItemChangedCallback, null)
            });
            for (int i = 0; i < loopCount; i++)
            {
                AssertUtilities.WriteLine(i);
                IEasyDAClientTest_Com.ThreadsTest(InstantiateAndBrowseServers, threadCount);
            }
            client.UnsubscribeAllItems();
        }

        static private void ItemChangedCallback(object sender, EasyDAItemChangedEventArgs eventArgs)
        {
            AssertUtilities.WriteLine("Item changed");
        }

        #endregion



        #region Test Methods

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BackgroundWorkersTest_InstantiateAndBrowseServers_5()
        {
            IEasyDAClientTest_Com.BackgroundWorkersTest(InstantiateAndBrowseServers, 5);
        }

        [Ignore]    // IMPROVE: works OK but takes long - put back after being able to exclude the long-running tests easily
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BackgroundWorkersTest_InstantiateAndBrowseServers_WhileSubscribed_2_Looping_100()
        {
            const int loopCount = 100;
            BackgroundWorkers_InstantiateAndBrowseServers_WhileSubscribed(2, loopCount);
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ThreadsTest_InstantiateAndBrowseServers_5()
        {
            IEasyDAClientTest_Com.ThreadsTest(InstantiateAndBrowseServers, 5);
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ThreadsTest_InstantiateAndBrowseServers_WhileSubscribed_2()
        {
            Threads_InstantiateAndBrowseServers_WhileSubscribed(2);
        }

        [Ignore]    // IMPROVE: works OK but takes long - put back after being able to exclude the long-running tests easily
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ThreadsTest_InstantiateAndBrowseServers_WhileSubscribed_2_Looping_100()
        {
            const int loopCount = 100;
            Threads_InstantiateAndBrowseServers_WhileSubscribed(2, loopCount);
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ThreadsTest_InstantiateAndBrowseServers_WhileSubscribed_5()
        {
            Threads_InstantiateAndBrowseServers_WhileSubscribed(5);
        }

        [Ignore]    // IMPROVE: put back; is long-running
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ThreadsTest_InstantiateAndBrowseServers_WhileSubscribed_10()
        {
            Threads_InstantiateAndBrowseServers_WhileSubscribed(10);
        }

        #endregion
    }
}
