﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Threading;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.DataAccess
{
    // ReSharper disable once InconsistentNaming
    public partial class IEasyDAClientTest_Com
    {

        static public void BackgroundWorkersTest([NotNull] Action action, int count)
        {
            var countdownEvent = new CountdownEvent(count);

            var backgroundWorkerArray = new BackgroundWorker[count];
            for (int i = 0; i < count; i++)
            {
                var backgroundWorker = new BackgroundWorker();
                backgroundWorker.DoWork += (sender, args) =>
                {
                    AssertUtilities.WriteLine("Action...");
                    try
                    {
                        action();
                    }
                    catch (Exception exception)
                    {
                        AssertUtilities.WriteLine(exception);
                    }
                    AssertUtilities.WriteLine("Finished.");

                    countdownEvent.Signal();
                };
                backgroundWorkerArray[i] = backgroundWorker;
            }

            for (int i = 0; i < count; i++)
            {
                Debug.Assert(backgroundWorkerArray[i] != null);
                backgroundWorkerArray[i].RunWorkerAsync();
            }

            countdownEvent.Wait();
        }

        static public void ThreadsTest([NotNull] Action action, int count)
        {
            var countdownEvent = new CountdownEvent(count);

            var threadArray = new Thread[count];
            for (int i = 0; i < count; i++)
            {
                var thread = new Thread(() =>
                {
                    AssertUtilities.WriteLine("Action...");
                    try
                    {
                        action();
                    }
                    catch (Exception exception)
                    {
                        AssertUtilities.WriteLine(exception);
                    }
                    AssertUtilities.WriteLine("Finished.");

                    countdownEvent.Signal();
                });
                threadArray[i] = thread;
            }

            for (int i = 0; i < count; i++)
            {
                Debug.Assert(threadArray[i] != null);
                threadArray[i].Start();
            }

            countdownEvent.Wait();
        }



        private void InstantiateAndReadItem()
        {
            var client = CreateClient();
            AssertUtilities.WriteLine(client.ReadItem(LegacyTestDefs.ProgIds.KitServer, "Simulation.Random"));
        }




        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BackgroundWorkersTest_InstantiateAndReadItem_100()
        {
            BackgroundWorkersTest(InstantiateAndReadItem, 100);
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        // ReSharper disable once InconsistentNaming
        public void BackgroundWorkersTest_InstantiateAndReadItem_100_GC()
        {
            BackgroundWorkersTest_InstantiateAndReadItem_100();
            GC.Collect();
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void DisposeFromDifferentThreadTest1()
        {
            var client = CreateClient();
            var disposable = (IDisposable)client;
            AssertUtilities.WriteLine(client.ReadItem(LegacyTestDefs.ProgIds.KitServer, "Simulation.Random"));

            var thread = new Thread(() =>
            {
                AssertUtilities.WriteLine("Disposing...");
                disposable.Dispose();
            });
            thread.Start();

            thread.Join();
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void DisposeFromDifferentThreadTest2()
        {
            IEasyDAClient client = null;
            var thread = new Thread(() =>
            {
                client = CreateClient();
                AssertUtilities.WriteLine(client.ReadItem(LegacyTestDefs.ProgIds.KitServer, "Simulation.Random"));
            });
            thread.Start();

            thread.Join();
            Debug.Assert(client != null);

            var disposable = (IDisposable)client;
            AssertUtilities.WriteLine("Disposing...");
            disposable.Dispose();
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ThreadsTest_InstantiateAndReadItem_100()
        {
            ThreadsTest(InstantiateAndReadItem, 100);
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        // ReSharper disable once InconsistentNaming
        public void ThreadsTest_InstantiateAndReadItem_100_GC()
        {
            ThreadsTest_InstantiateAndReadItem_100();
            GC.Collect();
        }
    }
}
