﻿
using System.Collections.Generic;
using JetBrains.Annotations;
using OpcLabs.EasyOpc.UA;

namespace OpcLabs.EasyOpcLiveTest.UA.Environment
{
    // ReSharper disable once InconsistentNaming
    public class OpcFoundation_UASampleServer : UAServer
    {
        static public class BrowsePathStrings
        {
            // ReSharper disable InconsistentNaming
            //public const string Data_Dynamic_UserScalar_Int32Value = "[ObjectsFolder]/Data/Dynamic/UserScalar/Int32Value";
            public const string Data_Static_UserScalar_Int32Value = "[ObjectsFolder]/Data/Static/UserScalar/Int32Value";
            // ReSharper restore InconsistentNaming
        }

        public class EndpointDescriptorsInfo
        {
            public EndpointDescriptorsInfo([NotNull] OpcFoundation_UASampleServer parent)
            {
                _parent = parent;
            }

            [NotNull]
            public UAEndpointDescriptor Http => new UAEndpointDescriptor(_parent._endpointUrlStrings.Http);

            [NotNull]
            public UAEndpointDescriptor Https => new UAEndpointDescriptor(_parent._endpointUrlStrings.Https);

            [NotNull]
            public UAEndpointDescriptor OpcTcp => new UAEndpointDescriptor(_parent._endpointUrlStrings.OpcTcp);

            [NotNull]
            private readonly OpcFoundation_UASampleServer _parent;
        }

        static public class NamespaceUriStrings
        {
            public const string Default = "http://test.org/UA/Data/";

        }

        static public class NodeIdExpandedTexts
        {
            // ReSharper disable InconsistentNaming
            public const string DataType_ScalarValueDataType = "nsu=http://test.org/UA/Data/;i=9440";
            public const string DataType_ArrayValueDataType = "nsu=http://test.org/UA/Data/;i=9669";

            public const string EncodingId_ScalarValueDataType_DefaultBinary = "nsu=http://test.org/UA/Data/;i=11437";
            public const string EncodingId_ArrayValueDataType_DefaultBinary = "nsu=http://test.org/UA/Data/;i=11438";

            public const string Data_Dynamic_Scalar_SByteValue = "nsu=http://test.org/UA/Data/;i=10845";
            public const string Data_Dynamic_Scalar_FloatValue = "nsu=http://test.org/UA/Data/;i=10853";
            public const string Data_Dynamic_UserScalar_Int32Value = "nsu=http://test.org/UA/Data/;i=11017";

            public const string Data_Static_Array_UInt16Value = "nsu=http://test.org/UA/Data/;i=10304";
            public const string Data_Static_MethodTest = "nsu=http://test.org/UA/Data/;i=10755";
            public const string Data_Static_MethodTest_ScalarMethod1 = "nsu=http://test.org/UA/Data/;i=10756";
            public const string Data_Static_Scalar_Int32Value = "nsu=http://test.org/UA/Data/;i=10221";
            public const string Data_Static_Scalar_StructureValue = "nsu=http://test.org/UA/Data/;i=10239";
            public const string Data_Static_UserArray_UInt16Value = "nsu=http://test.org/UA/Data/;i=10467";
            public const string Data_Static_UserScalar = "nsu=http://test.org/UA/Data/;i=10327";
            public const string Data_Static_UserScalar_GenerateValues = "nsu=http://test.org/UA/Data/;i=10329";
            public const string Data_Static_UserScalar_UInt16Value = "nsu=http://test.org/UA/Data/;i=10388";    // reserved for quality tests only!
            public const string Data_Static_UserScalar_Int32Value = "nsu=http://test.org/UA/Data/;i=10389";

            public const string MemoryBuffers_UInt32Format = "nsu=http://samples.org/UA/memorybuffer/Instance;s=UInt32[{0}]";

            public const string Unknown = "nsu=http://test.org/UA/Data/;i=999888777";
            // ReSharper restore InconsistentNaming
        }



        private class EndpointUrlStringsInfo
        {
            public EndpointUrlStringsInfo([NotNull] OpcFoundation_UASampleServer parent)
            {
                _parent = parent;
            }

            [NotNull]
            public string Http => "http://" + _parent.HostName + ":51211/UA/SampleServer";
            [NotNull]
            public string Https => "https://" + _parent.HostName + ":51212/UA/SampleServer/";
            [NotNull]
            public string OpcTcp => "opc.tcp://" + _parent.HostName + ":51210/UA/SampleServer";

            [NotNull]
            private readonly OpcFoundation_UASampleServer _parent;
        }



        public OpcFoundation_UASampleServer([NotNull] string hostName) 
            : base(hostName)
        {
            EndpointDescriptors = new EndpointDescriptorsInfo(this);
            _endpointUrlStrings = new EndpointUrlStringsInfo(this);
        }

        [NotNull]
        public readonly EndpointDescriptorsInfo EndpointDescriptors;

        [NotNull]
        private readonly EndpointUrlStringsInfo _endpointUrlStrings;

        public override IEnumerable<UAEndpointDescriptor> AllEndpointDescriptors
        {
            get
            {
                yield return EndpointDescriptors.OpcTcp;
                yield return EndpointDescriptors.Http;
                yield return EndpointDescriptors.Https;
            }
        }
    }
}
