﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Collections.Specialized;
using OpcLabs.EasyOpc.DataAccess.OperationModel;
using OpcLabs.EasyOpcFactory.DataAccess;

namespace OpcLabs.EasyOpcTest.DataAccess.OperationModel
{
    public partial class DAVtqResultTest
    {
        #region Test Methods
        // ReSharper disable InconsistentNaming

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_F_Failure_ErrorId_Test()
        {
            Assert.AreEqual("*** Failure -2: Some error [...] (0 diagnostics); ", new DAVtqResult { ErrorMessage = "Some error\nSecond line", ErrorId = "-2" }.ToString("F"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_F_Failure_NoErrorId_Test()
        {
            Assert.AreEqual("*** Failure: Some error [...] (0 diagnostics); ", new DAVtqResult { ErrorMessage = "Some error\nSecond line" }.ToString("F"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_F_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success:  (4 diagnostics); 3.141 {System.Double} @1967-08-27T23:30:00.000; GoodNonspecific (192)",
                new DAVtqResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList()),
                    Vtq = DAVtqInstances.ADouble
                }.ToString("F"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_F_Success_ErrorId_Test()
        {
            Assert.AreEqual(
                "Success:  (0 diagnostics); 3.141 {System.Double} @1967-08-27T23:30:00.000; GoodNonspecific (192)", 
                new DAVtqResult { ErrorId = "3", Vtq = DAVtqInstances.ADouble }.ToString("F"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_F_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual(
                "Success:  (0 diagnostics); 3.141 {System.Double} @1967-08-27T23:30:00.000; GoodNonspecific (192)", 
                new DAVtqResult { Vtq = DAVtqInstances.ADouble }.ToString("F"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_F_Success_Null_Test()
        {
            Assert.AreEqual("Success:  (0 diagnostics); ", new DAVtqResult().ToString("F"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_g_Failure_ErrorId_Test()
        {
            Assert.AreEqual("*** Failure -2: Some error [...]", new DAVtqResult { ErrorMessage = "Some error\nSecond line", ErrorId = "-2" }.ToString("g"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_g_Failure_NoErrorId_Test()
        {
            Assert.AreEqual("*** Failure: Some error [...]", new DAVtqResult { ErrorMessage = "Some error\nSecond line" }.ToString("g"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_g_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success; 3.141 @1967-08-27T23:30:00.000; GoodNonspecific (192)",
                new DAVtqResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList()),
                    Vtq = DAVtqInstances.ADouble
                }.ToString("g"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_g_Success_ErrorId_Test()
        {
            Assert.AreEqual(
                "Success 3; 3.141 @1967-08-27T23:30:00.000; GoodNonspecific (192)", 
                new DAVtqResult { ErrorId = "3", Vtq = DAVtqInstances.ADouble }.ToString("g"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_g_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual("Success; 3.141 @1967-08-27T23:30:00.000; GoodNonspecific (192)", new DAVtqResult { Vtq = DAVtqInstances.ADouble }.ToString("g"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void Formatting_g_Success_Null_Test()
        {
            Assert.AreEqual("Success; ", new DAVtqResult().ToString("g"));
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ToString_Failure_ErrorId_Test()
        {
            Assert.AreEqual("*** Failure -2 (0xFFFFFFFE): Some error [...]", new DAVtqResult { ErrorMessage = "Some error\nSecond line", ErrorId = "-2" }.ToString());
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ToString_Failure_NoErrorId_Test()
        {
            Assert.AreEqual("*** Failure: Some error [...]", new DAVtqResult { ErrorMessage = "Some error\nSecond line" }.ToString());
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ToString_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success (4 diagnostics); 3.141 {System.Double} @1967-08-27T23:30:00.000; GoodNonspecific (192)",
                new DAVtqResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList()),
                    Vtq = DAVtqInstances.ADouble
                }.ToString());
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ToString_Success_ErrorId_Test()
        {
            Assert.AreEqual(
                "Success 3 (0x00000003); 3.141 {System.Double} @1967-08-27T23:30:00.000; GoodNonspecific (192)", 
                new DAVtqResult { ErrorId = "3", Vtq = DAVtqInstances.ADouble }.ToString());
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ToString_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual(
                "Success; 3.141 {System.Double} @1967-08-27T23:30:00.000; GoodNonspecific (192)", 
                new DAVtqResult { Vtq = DAVtqInstances.ADouble }.ToString());
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ToString_Success_Null_Test()
        {
            Assert.AreEqual("Success; ", new DAVtqResult().ToString());
        }

        // ReSharper restore InconsistentNaming
        #endregion
    }
}
