﻿using System;
using System.Collections;
using System.Linq;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.DataTypeModel;
using OpcLabs.BaseLib.Extensions.Internal;
using OpcLabs.BaseLibFactory;
using OpcLabs.EasyOpc.UA.DataTypeModel;
using OpcLabs.EasyOpc.UA.DataTypeModel.Implementation;

namespace OpcLabs.EasyOpcTest.UA.DataTypeModel.Internal
{
    partial class TypeDescriptionExamplesTest
    {
        #region Static Private Properties

        [NotNull]
        static private StructuredDataType VariantDataType
        {
            get
            {
                var structuredDataType = new StructuredDataType("Variant", "TypeDescriptionExamples:Variant")
                {
                    new DataField("ArrayLengthSpecified", Variant_ArrayLengthSpecifiedDataType) {IsSwitch = true},
                    new DataField("VariantType", Variant_VariantTypeDataType) {IsSwitch = true},
                    new DataField("ArrayLength", UAOpcBinaryStandardDataTypes.Int32) {IsLength = true, IsOptional = true},
                    new DataField("Int32", Variant_Int32DataType) {IsOptional = true},
                    new DataField("String", Variant_StringDataType) {IsOptional = true},
                    new DataField("DateTime", Variant_DateTimeDataType) {IsOptional = true}
                };
                structuredDataType.Description = "A union of several types.";
                return structuredDataType;
            }
        }

        [NotNull]
        static private SequenceDataType Variant_ArrayLengthSpecifiedDataType => new SequenceDataType(1, UAOpcBinaryStandardDataTypes.Bit);
        [NotNull]
        static private SequenceDataType Variant_VariantTypeDataType => new SequenceDataType(7, UAOpcBinaryStandardDataTypes.Bit);
        [NotNull]
        static private SequenceDataType Variant_Int32DataType => new SequenceDataType(UAOpcBinaryStandardDataTypes.Int32);
        [NotNull]
        static private SequenceDataType Variant_StringDataType => new SequenceDataType(UAOpcBinaryStandardDataTypes.String);
        [NotNull]
        static private SequenceDataType Variant_DateTimeDataType => new SequenceDataType(UAOpcBinaryStandardDataTypes.DateTime);

        [NotNull]
        static private StructuredData Variant_NoneData => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:7)}
        };

        [NotNull]
        static private byte[] Variant_NoneExtensionObjectBody => new byte[1];

        [NotNull]
        static private StructuredData Variant_None_ArrayLengthSpecifiedData => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:1, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:7)},
            {"ArrayLength", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 85)}
        };

        [NotNull]
        static private byte[] Variant_None_ArrayLengthSpecifiedExtensionObjectBody => new byte[] { 1, 0x55, 0, 0, 0 };

        [NotNull]
        static private StructuredData Variant_Int32Data => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:1, bitCount:7)},
            {"Int32", SequenceData.FromPrimitiveValues(Variant_Int32DataType, UAOpcBinaryStandardDataTypes.Int32, new [] {unchecked((Int32)0x87654321)})}
        };

        [NotNull]
        static private byte[] Variant_Int32ExtensionObjectBody => new byte[] { 1 << 1, 0x21, 0x43, 0x65, 0x87 };

        [NotNull]
        static private StructuredData Variant_Int32_ArrayLengthSpecifiedData => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:1, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:1, bitCount:7)},
            {"Int32", SequenceData.FromPrimitiveValues(Variant_Int32DataType, UAOpcBinaryStandardDataTypes.Int32, new [] {unchecked((Int32)0x87654321)})}
        };

        [NotNull]
        static private byte[] Variant_Int32_ArrayLengthSpecifiedExtensionObjectBody => new byte[] { 1 | (1 << 1), 1, 0, 0, 0, 0x21, 0x43, 0x65, 0x87 };

        // auto-complete will fill in the ArrayLength as 3
        [NotNull]
        static private StructuredData Variant_Int32_ArrayLengthSpecified_MoreThanOneElementData => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:1, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:1, bitCount:7)},
            {"Int32", SequenceData.FromPrimitiveValues(Variant_Int32DataType, UAOpcBinaryStandardDataTypes.Int32, new []
                {unchecked((Int32)0x87654321), 0x23456789, unchecked((Int32)0xAAAAAAAA)})}
        };

        [NotNull]
        static private byte[] Variant_Int32_ArrayLengthSpecified_MoreThanOneElementDataExtensionObjectBody => new byte[] { 1 | (1 << 1), 3, 0, 0, 0, 0x21, 0x43, 0x65, 0x87, 0x89, 0x67, 0x45, 0x23, 0xAA, 0xAA, 0xAA, 0xAA };

        [NotNull]
        static private StructuredData Variant_Int32_ArrayLengthSpecified_NegativeData => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:1, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:1, bitCount:7)},
            {"ArrayLength", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, -1) },
            {"Int32", SequenceData.FromPrimitiveValues(Variant_Int32DataType, UAOpcBinaryStandardDataTypes.Int32, new [] {unchecked((Int32)0x87654321)})}
        };

        // IMPROVE: support auto-completing this
        //[NotNull]
        //static private StructuredData Variant_Int32_ArrayLengthSpecified_NoSequenceData => new StructuredData(VariantDataType)
        //{
        //    {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, BitDataType, value:1, bitCount:1)},
        //    {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, BitDataType, value:1, bitCount:7)},
        //};

        [NotNull]
        static private StructuredData Variant_Int32_ArrayLengthSpecified_WithStringData => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:1, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:1, bitCount:7)},
            {"Int32", SequenceData.FromPrimitiveValues(Variant_Int32DataType, UAOpcBinaryStandardDataTypes.Int32, new [] {unchecked((Int32)0x87654321)})},
            {"String", SequenceData.FromPrimitiveValues(Variant_StringDataType, UAOpcBinaryStandardDataTypes.String, new [] {"abc", "def"})}
        };

        [NotNull]
        static private byte[] Variant_Int32_ArrayLengthSpecified_NegativeExtensionObjectBody => new byte[] { 1 | (1 << 1), 0xFF, 0xFF, 0xFF, 0xFF };

        // this cannot be auto-completed, because ArrayLengthSpecified is False
        [NotNull]
        static private StructuredData Variant_Int32_MoreThanOneElementData => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:1, bitCount:7)},
            {"Int32", SequenceData.FromPrimitiveValues(Variant_Int32DataType, UAOpcBinaryStandardDataTypes.Int32, new []
                {unchecked((Int32)0x87654321), 0x23456789, unchecked((Int32)0xAAAAAAAA)})}
        };

        [NotNull]
        static private StructuredData Variant_StringData => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:2, bitCount:7)},
            {"String", SequenceData.FromPrimitiveValues(Variant_StringDataType, UAOpcBinaryStandardDataTypes.String, new [] {"test"})}
        };

        [NotNull]
        static private byte[] Variant_StringExtensionObjectBody => (new byte[] { 2 << 1 })
            .Concat("test".ToByteArray())
            .Concat(new byte[1])
            .ToArray();

        [NotNull]
        static private StructuredData Variant_String_EmptyData => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:2, bitCount:7)},
            {"String", SequenceData.FromPrimitiveValues(Variant_StringDataType, UAOpcBinaryStandardDataTypes.String, new [] {""})}
        };

        [NotNull]
        static private byte[] Variant_String_EmptyExtensionObjectBody => new byte[] { 2 << 1, 0 };

        [NotNull]
        static private StructuredData Variant_String_WithWideCharData => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:2, bitCount:7)},
            {"String", SequenceData.FromPrimitiveValues(Variant_StringDataType, UAOpcBinaryStandardDataTypes.String, new [] {"a\u1234z"})}
        };

        [NotNull]
        static private byte[] Variant_String_WithWideCharDataExtensionObjectBody => (new byte[] { 2 << 1 })
            .Concat(new byte[] { 0x61, 0xE1, 0x88, 0xB4, 0x7A })
            .Concat(new byte[1])
            .ToArray();

        [NotNull]
        static private StructuredData Variant_String_WithTerminator_1Data => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:2, bitCount:7)},
            {"String", SequenceData.FromPrimitiveValues(Variant_StringDataType, UAOpcBinaryStandardDataTypes.String, new [] {"\x0"})}
        };

        [NotNull]
        static private StructuredData Variant_String_WithTerminator_2Data => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:2, bitCount:7)},
            {"String", SequenceData.FromPrimitiveValues(Variant_StringDataType, UAOpcBinaryStandardDataTypes.String, new [] {"\x0test"})}
        };

        [NotNull]
        static private StructuredData Variant_String_WithTerminator_3Data => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:2, bitCount:7)},
            {"String", SequenceData.FromPrimitiveValues(Variant_StringDataType, UAOpcBinaryStandardDataTypes.String, new [] {"test\x0"})}
        };

        [NotNull]
        static private StructuredData Variant_String_WithTerminator_4Data => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:2, bitCount:7)},
            {"String", SequenceData.FromPrimitiveValues(Variant_StringDataType, UAOpcBinaryStandardDataTypes.String, new [] {"test\x0test"})}
        };

        [NotNull]
        static private StructuredData Variant_DateTimeData => new StructuredData(VariantDataType)
        {
            {"ArrayLengthSpecified", SequenceData.BitsFromValue(Variant_ArrayLengthSpecifiedDataType, UAOpcBinaryStandardDataTypes.Bit, value:0, bitCount:1)},
            {"VariantType", SequenceData.BitsFromValue(Variant_VariantTypeDataType, UAOpcBinaryStandardDataTypes.Bit, value:3, bitCount:7)},
            {"DateTime", SequenceData.FromPrimitiveValues(Variant_DateTimeDataType, UAOpcBinaryStandardDataTypes.DateTime, new [] { DateTimeUtcInstances.Kind_Utc })}
        };

        [NotNull]
        static private byte[] Variant_DateTimeExtensionObjectBody => new byte[] { 3 << 1, 0x00, 0x8C, 0x58, 0xA8, 0x6B, 0x10, 0x9B, 0x01 };

        #endregion



        #region Test Methods

        [TestMethod]
        public void DecodeTest_Variant_DateTime() =>
            // ReSharper disable once PossibleNullReferenceException
            DecodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(), 
                TypeDictionaryByteArray, "Variant", Variant_DateTimeExtensionObjectBody, Variant_DateTimeData);

        [TestMethod]
        public void DecodeTest_Variant_None() =>
            // ReSharper disable once PossibleNullReferenceException
            DecodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(), 
                TypeDictionaryByteArray, "Variant", Variant_NoneExtensionObjectBody, Variant_NoneData);

        [TestMethod]
        public void DecodeTest_Variant_None_ArrayLengthSpecified() =>
            // ReSharper disable once PossibleNullReferenceException
            DecodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(), 
                TypeDictionaryByteArray, "Variant", Variant_None_ArrayLengthSpecifiedExtensionObjectBody, Variant_None_ArrayLengthSpecifiedData);

        [TestMethod]
        public void DecodeTest_Variant_Int32() =>
            // ReSharper disable once PossibleNullReferenceException
            DecodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(), 
                TypeDictionaryByteArray, "Variant", Variant_Int32ExtensionObjectBody, Variant_Int32Data);

        [TestMethod]
        public void DecodeTest_Variant_Int32_ArrayLengthSpecified() =>
            // ReSharper disable once PossibleNullReferenceException
            DecodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(),
                TypeDictionaryByteArray,
                "Variant",
                Variant_Int32_ArrayLengthSpecifiedExtensionObjectBody,
                // ReSharper disable once PossibleNullReferenceException
                new StructuredData(Variant_Int32_ArrayLengthSpecifiedData)
                    .Apply(data => data.Add("ArrayLength", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 1))));

        [TestMethod]
        public void DecodeTest_Variant_Int32_ArrayLengthSpecified_Negative() =>
            // ReSharper disable once PossibleNullReferenceException
            DecodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(),
                TypeDictionaryByteArray,
                "Variant",
                Variant_Int32_ArrayLengthSpecified_NegativeExtensionObjectBody,
                new StructuredData(Variant_Int32_ArrayLengthSpecified_NegativeData)
                    // ReSharper disable once PossibleNullReferenceException
                    .Apply(data => data.Remove("Int32")));

        [TestMethod]
        public void DecodeTest_Variant_String() =>
            // ReSharper disable once PossibleNullReferenceException
            DecodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(), 
                TypeDictionaryByteArray, "Variant", Variant_StringExtensionObjectBody, Variant_StringData);

        [TestMethod]
        public void DecodeTest_Variant_String_Empty() =>
            // ReSharper disable once PossibleNullReferenceException
            DecodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(), 
                TypeDictionaryByteArray, "Variant", Variant_String_EmptyExtensionObjectBody, Variant_String_EmptyData);

        [TestMethod]
        public void DecodeTest_Variant_String_WithWideChar() =>
            // ReSharper disable once PossibleNullReferenceException
            DecodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(), 
                TypeDictionaryByteArray, "Variant", Variant_String_WithWideCharDataExtensionObjectBody, Variant_String_WithWideCharData);

        [TestMethod]
        public void EncodeErrorTest_BinaryEncoderSequenceDataActualLengthMismatch_2()
        {
            var structuredData = new StructuredData
            {
                {"ArrayLengthSpecified", new SequenceData { new PrimitiveData(UAOpcBinaryStandardDataTypes.Bit, false) }},
                {"VariantType", new SequenceData { new OpaqueData(new BitArray(7)) }}
            };
            EncodeErrorTest(null, 
                TypeDictionaryByteArray,
                "Variant",
                structuredData,
                UADataTypeModelErrorCodes.OpcBinaryEncoderSequenceDataActualLengthMismatch);
        }

        [TestMethod]
        public void EncodeErrorTest_BinaryEncoderSequenceDataElementMatchesTerminator_1() =>
            // ReSharper disable once PossibleNullReferenceException
            EncodeErrorTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(), 
                TypeDictionaryByteArray,
                "Variant",
                Variant_String_WithTerminator_1Data,
                UADataTypeModelErrorCodes.OpcBinaryEncoderSequenceDataElementMatchesTerminator);

        [TestMethod]
        public void EncodeErrorTest_BinaryEncoderSequenceDataElementMatchesTerminator_2() =>
            // ReSharper disable once PossibleNullReferenceException
            EncodeErrorTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(),
                TypeDictionaryByteArray,
                "Variant",
                Variant_String_WithTerminator_2Data,
                UADataTypeModelErrorCodes.OpcBinaryEncoderSequenceDataElementMatchesTerminator);

        [TestMethod]
        public void EncodeErrorTest_BinaryEncoderSequenceDataElementMatchesTerminator_3() =>
            // ReSharper disable once PossibleNullReferenceException
            EncodeErrorTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(),
                TypeDictionaryByteArray,
                "Variant",
                Variant_String_WithTerminator_3Data,
                UADataTypeModelErrorCodes.OpcBinaryEncoderSequenceDataElementMatchesTerminator);

        [TestMethod]
        public void EncodeErrorTest_BinaryEncoderSequenceDataElementMatchesTerminator_4() =>
            // ReSharper disable once PossibleNullReferenceException
            EncodeErrorTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(),
                TypeDictionaryByteArray,
                "Variant",
                Variant_String_WithTerminator_4Data,
                UADataTypeModelErrorCodes.OpcBinaryEncoderSequenceDataElementMatchesTerminator);

        [TestMethod]
        public void EncodeErrorTest_BinaryEncoderSequenceFieldNotSequenceData_2()
        {
            var structuredData = new StructuredData
            {
                {"ArrayLengthSpecified", new PrimitiveData(UAOpcBinaryStandardDataTypes.Bit, false) }
            };
            EncodeErrorTest(null, 
                TypeDictionaryByteArray,
                "Variant",
                structuredData,
                UADataTypeModelErrorCodes.OpcBinaryEncoderSequenceFieldNotSequenceData);
        }

        [TestMethod]
        public void EncodeErrorTest_BinaryEncoderSequenceNonDefaultLengthFieldNotPresent() =>
            EncodeErrorTest(null, 
                TypeDictionaryByteArray,
                "Variant",
                Variant_Int32_MoreThanOneElementData,
                UADataTypeModelErrorCodes.OpcBinaryEncoderSequenceNonDefaultLengthFieldNotPresent);

        [TestMethod]
        public void EncodeTest_Variant_DateTime() =>
            EncodeTest(null, TypeDictionaryByteArray, "Variant", Variant_DateTimeData, Variant_DateTimeExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_Variant_None() =>
            EncodeTest(null, TypeDictionaryByteArray, "Variant", Variant_NoneData, Variant_NoneExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_Variant_None_ArrayLengthSpecified() =>
            EncodeTest(null, TypeDictionaryByteArray, "Variant", Variant_None_ArrayLengthSpecifiedData, Variant_None_ArrayLengthSpecifiedExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_Variant_Int32() =>
            EncodeTest(null, TypeDictionaryByteArray, "Variant", Variant_Int32Data, Variant_Int32ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_Variant_Int32_ArrayLengthSpecified() =>
            EncodeTest(null, TypeDictionaryByteArray, "Variant", Variant_Int32_ArrayLengthSpecifiedData, Variant_Int32_ArrayLengthSpecifiedExtensionObjectBody);

        // tests that auto-complete will override any non-negative length
        [TestMethod]
        public void EncodeTest_Variant_Int32_ArrayLengthSpecified_AutoCompleteOverride() =>
            EncodeTest(null, 
                TypeDictionaryByteArray,
                "Variant",
                new StructuredData(Variant_Int32_ArrayLengthSpecifiedData)
                    // ReSharper disable once PossibleNullReferenceException
                    .Apply(data => data.Add("ArrayLength", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 85))),
                Variant_Int32_ArrayLengthSpecifiedExtensionObjectBody);

        // tests that auto-complete will not override a negative length
        [TestMethod]
        public void EncodeTest_Variant_Int32_ArrayLengthSpecified_AutoCompleteNotOverride() =>
            EncodeTest(null, TypeDictionaryByteArray, "Variant", Variant_Int32_ArrayLengthSpecified_NegativeData, Variant_Int32_ArrayLengthSpecified_NegativeExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_Variant_Int32_ArrayLengthSpecified_MoreThanOneElementData() =>
            EncodeTest(null, TypeDictionaryByteArray, "Variant", Variant_Int32_ArrayLengthSpecified_MoreThanOneElementData, Variant_Int32_ArrayLengthSpecified_MoreThanOneElementDataExtensionObjectBody);

        // the auto-complete must not get confused by the length of a non-selected variant (String)
        [TestMethod]
        public void EncodeTest_Variant_Int32_ArrayLengthSpecified_WithString() =>
            EncodeTest(null, TypeDictionaryByteArray, "Variant", Variant_Int32_ArrayLengthSpecified_WithStringData, Variant_Int32_ArrayLengthSpecifiedExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_Variant_String() =>
            // ReSharper disable once PossibleNullReferenceException
            EncodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(),
                TypeDictionaryByteArray, "Variant", Variant_StringData, Variant_StringExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_Variant_String_Empty() =>
            // ReSharper disable once PossibleNullReferenceException
            EncodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(),
                TypeDictionaryByteArray, "Variant", Variant_String_EmptyData, Variant_String_EmptyExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_Variant_String_WithWideChar() =>
            // ReSharper disable once PossibleNullReferenceException
            EncodeTest(dts => dts.Parameters.TypeNameRemappingDictionary.Clear(),
                TypeDictionaryByteArray, "Variant", Variant_String_WithWideCharData, Variant_String_WithWideCharDataExtensionObjectBody);

        #endregion
    }
}
