﻿using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace;
using OpcLabs.UnitTestLib;

namespace OpcLabs.OpcInteroperabilityTest.UA.Servers
{
    [TestClass]
    // ReSharper disable once InconsistentNaming
    public class YokogawaElectricCorporation_FastToolsOpcUAServerServer_Test : UAInteroperabilityWorkshopTest
    {
        public override IEnumerable<UANodeDescriptor> DynamicNodes => new UANodeDescriptor[]
        {
            "nsu=urn:Yokogawa:FASTTOOLS_UA_SERVER ;ns=2;s={\"0\":\"Value\",\"nsid\":\"12\"}",
            "nsu=urn:Yokogawa:FASTTOOLS_UA_SERVER ;ns=2;s={\"0\":\"Value\",\"nsid\":\"26\"}",
        };

        public override IEnumerable<UAEndpointDescriptor> EndpointDescriptors => new UAEndpointDescriptor[] { "opc.tcp://FTSCADA:34493" };

        public override string Password => "opc";

        public override IEnumerable<string> ServerNotSupportedTests => base.ServerNotSupportedTests.Concat(new[]
        {
            nameof(Group001_Session_Test02_CreateSession_UaTcp_Basic128Rsa15),
            nameof(Group001_Session_Test09_CreateSession_Https_None),
            nameof(Group001_Session_Test10_ActivateSession_X509Certificate),
            //nameof(Group002_ViewServices_Test04_Browse_NodeClasses),  //?
            nameof(Group003_AttributeServices_Test05_Read_Array_IndexRange),
            nameof(Group003_AttributeServices_Test07_Write_Array_IndexRange),

            //nameof(Group102_Session2_Test01_CreateSession_UaTcp_Basic256),
            //nameof(Group102_Session2_Test02_CreateSession_UaTcp_Basic256Sha256),
            //nameof(Group102_Session2_Test03_CreateSession_Https_Basic256Sha256)
        });

        public override IEnumerable<UANodeDescriptor> StaticNodes => new UANodeDescriptor[] { "nsu=urn:Yokogawa:FASTTOOLS_UA_SERVER ;ns=2;s={\"0\":\"Value\",\"nsid\":\"24\"}" };

        public override string UserName => "opc";

        //

        [TestMethod]
        public void BrowseDataNodesTest()
        {
            TestSection.Arrange();
            PrecheckTestMethod();
            UAEndpointDescriptor endpointDescriptor = EndpointDescriptorCommon;

            TestSection.Act();
            using (var client = new EasyUAClient { Isolated = true })
            {
                UANodeElementCollection nodeElementCollection = client.BrowseDataNodes(endpointDescriptor,
                    "nsu=urn:Yokogawa:FASTTOOLS_UA_SERVER ;ns=2;s={\"nsid\":\"12\"}");
                foreach (UANodeElement nodeElement in nodeElementCollection)
                    // ReSharper disable once PossibleNullReferenceException
                    AssertUtilities.WriteLine($"{nodeElement.DisplayName}: {nodeElement}");
                AssertUtilities.WriteLine($"Element count: {nodeElementCollection.Count}");

                TestSection.Assert();
                Assert.IsTrue(nodeElementCollection.Count >= 1, "Not enough elements");
            }
        }

        [TestMethod]
        public void BrowseVariablesTest()
        {
            TestSection.Arrange();
            PrecheckTestMethod();
            UAEndpointDescriptor endpointDescriptor = EndpointDescriptorCommon;

            TestSection.Act();
            using (var client = new EasyUAClient { Isolated = true })
            {
                UANodeElementCollection nodeElementCollection = client.BrowseVariables(endpointDescriptor,
                    "nsu=urn:Yokogawa:FASTTOOLS_UA_SERVER ;ns=2;s={\"nsid\":\"12\"}");
                foreach (UANodeElement nodeElement in nodeElementCollection)
                    // ReSharper disable once PossibleNullReferenceException
                    AssertUtilities.WriteLine($"{nodeElement.DisplayName}: {nodeElement}");
                AssertUtilities.WriteLine($"Element count: {nodeElementCollection.Count}");

                TestSection.Assert();
                Assert.IsTrue(nodeElementCollection.Count >= 1, "Not enough elements");
            }
        }
    }
}
